@@ -5,11 +5,11 @@
 extern "C" {
 #endif
 
-#define XMP_VERSION "4.2.8"
-#define XMP_VERCODE 0x040208
+#define XMP_VERSION "4.3.0"
+#define XMP_VERCODE 0x040300
 #define XMP_VER_MAJOR 4
-#define XMP_VER_MINOR 2
-#define XMP_VER_RELEASE 8
+#define XMP_VER_MINOR 3
+#define XMP_VER_RELEASE 0
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
 # if defined(BUILDING_STATIC)
@@ -23,6 +23,8 @@ extern "C" {
 # define EXPORT __attribute__((visibility ("default")))
 #elif defined(__SUNPRO_C)
 # define EXPORT __global
+#elif defined(EMSCRIPTEN)
+# define EXPORT EMSCRIPTEN_KEEPALIVE
 #else
 # define EXPORT 
 #endif
@@ -51,6 +53,7 @@ extern "C" {
 #define XMP_PLAYER_VOLUME	7	/* Player module volume */
 #define XMP_PLAYER_STATE	8	/* Internal player state */
 #define XMP_PLAYER_SMIX_VOLUME	9	/* SMIX volume */
+#define XMP_PLAYER_DEFPAN	10	/* Default pan setting */
 
 /* interpolation types */
 #define XMP_INTERP_NEAREST	0	/* Nearest neighbor */
@@ -313,6 +316,7 @@ EXPORT int         xmp_set_player      (xmp_context, int, int);
 EXPORT int         xmp_get_player      (xmp_context, int);
 EXPORT int         xmp_set_instrument_path (xmp_context, char *);
 EXPORT int         xmp_load_module_from_memory (xmp_context, void *, long);
+EXPORT int         xmp_load_module_from_file (xmp_context, void *, long);
 
 /* External sample mixer API */
 EXPORT int         xmp_start_smix       (xmp_context, int, int);
