@@ -10,7 +10,7 @@
 #include "xmp.h"
 
 #if defined(__GNUC__) || defined(__clang__)
-#if !defined(WIN32) && !defined(ANDROID) && !defined(__APPLE__) && !defined(__AMIGA__) && !defined(B_BEOS_VERSION) && !defined(__ATHEOS__)
+#if !defined(WIN32) && !defined(ANDROID) && !defined(__APPLE__) && !defined(__AMIGA__) && !defined(B_BEOS_VERSION) && !defined(__ATHEOS__) && !defined(EMSCRIPTEN) && !defined(__MINT__) 
 #define USE_VERSIONED_SYMBOLS
 #endif
 #endif
@@ -167,14 +167,18 @@ void __inline CLIB_DECL D_(const char *text, ...) { do {} while (0); }
 #define QUIRK_S3MRTG	(1 << 24)	/* S3M-style retrig when count == 0 */
 #define QUIRK_RTDELAY	(1 << 25)	/* Delay effect retrigs instrument */
 #define QUIRK_MLKDLY	(1 << 26)	/* MilkyTracker-style delay effect */
+#define QUIRK_ENVSUS	(1 << 27)	/* Key release jumps to sus point */
+#define QUIRK_S3MPMEM	(1 << 28)	/* S3M-style parameter memory */
+#define QUIRK_XMFINE	(1 << 29)	/* XM-style fine tune */
 
 #define HAS_QUIRK(x)	(m->quirk & (x))
 
 
 /* Format quirks */
 #define QUIRKS_ST3		(QUIRK_S3MLOOP | QUIRK_VOLPDN | QUIRK_FINEFX | \
-				 QUIRK_S3MLFO  | QUIRK_S3MRTG )
-#define QUIRKS_FT2		(QUIRK_RTDELAY | QUIRK_FINEFX )
+				 QUIRK_S3MPMEM | QUIRK_S3MRTG | QUIRK_S3MLFO )
+#define QUIRKS_FT2		(QUIRK_RTDELAY | QUIRK_FINEFX | QUIRK_ENVSUS | \
+				 QUIRK_XMFINE  )
 #define QUIRKS_IT		(QUIRK_S3MLOOP | QUIRK_FINEFX | QUIRK_VIBALL | \
 				 QUIRK_ENVFADE | QUIRK_ITVPOR | QUIRK_KEYOFF | \
 				 QUIRK_VIRTUAL | QUIRK_FILTER | QUIRK_S3MLFO | \
@@ -236,6 +240,7 @@ struct module_data {
 #define READ_EVENT_MED	4
 	int read_event_type;
 	int smpctl;			/* sample control flags */
+	int defpan;			/* default pan setting */
 	struct ord_data xxo_info[XMP_MAX_MOD_LENGTH];
 
 	int num_sequences;
