@@ -1,9 +1,23 @@
-/* Extended Module Player
+/* Extended Module Player core player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
- * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU Lesser General Public License. See COPYING.LIB
- * for more information.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
  */
 
 #include "common.h"
@@ -85,41 +99,6 @@ uint32 read32b(FILE *f)
 	return (a << 24) | (b << 16) | (c << 8) | d;
 }
 
-
-
-inline void write8(FILE *f, uint8 b)
-{
-	fputc(b, f);
-}
-
-void write16l(FILE *f, uint16 w)
-{
-	write8(f, w & 0x00ff);
-	write8(f, (w & 0xff00) >> 8);
-}
-
-void write16b(FILE *f, uint16 w)
-{
-	write8(f, (w & 0xff00) >> 8);
-	write8(f, w & 0x00ff);
-}
-
-void write32l(FILE *f, uint32 w)
-{
-	write8(f, w & 0x000000ff);
-	write8(f, (w & 0x0000ff00) >> 8);
-	write8(f, (w & 0x00ff0000) >> 16);
-	write8(f, (w & 0xff000000) >> 24);
-}
-
-void write32b(FILE *f, uint32 w)
-{
-	write8(f, (w & 0xff000000) >> 24);
-	write8(f, (w & 0x00ff0000) >> 16);
-	write8(f, (w & 0x0000ff00) >> 8);
-	write8(f, w & 0x000000ff);
-}
-
 uint16 readmem16l(uint8 *m)
 {
 	uint32 a, b;
@@ -186,6 +165,41 @@ uint32 readmem32b(uint8 *m)
 	return (a << 24) | (b << 16) | (c << 8) | d;
 }
 
+#ifndef LIBXMP_CORE_PLAYER
+
+inline void write8(FILE *f, uint8 b)
+{
+	fputc(b, f);
+}
+
+void write16l(FILE *f, uint16 w)
+{
+	write8(f, w & 0x00ff);
+	write8(f, (w & 0xff00) >> 8);
+}
+
+void write16b(FILE *f, uint16 w)
+{
+	write8(f, (w & 0xff00) >> 8);
+	write8(f, w & 0x00ff);
+}
+
+void write32l(FILE *f, uint32 w)
+{
+	write8(f, w & 0x000000ff);
+	write8(f, (w & 0x0000ff00) >> 8);
+	write8(f, (w & 0x00ff0000) >> 16);
+	write8(f, (w & 0xff000000) >> 24);
+}
+
+void write32b(FILE *f, uint32 w)
+{
+	write8(f, (w & 0xff000000) >> 24);
+	write8(f, (w & 0x00ff0000) >> 16);
+	write8(f, (w & 0x0000ff00) >> 8);
+	write8(f, w & 0x000000ff);
+}
+
 int move_data(FILE *out, FILE *in, int len)
 {
 	uint8 buf[1024];
@@ -200,3 +214,4 @@ int move_data(FILE *out, FILE *in, int len)
 	return 0;
 }
 
+#endif
