@@ -38,6 +38,31 @@
 	if ((rate) != 0) set_lfo_rate(lfo, rate); \
 } while (0)
 
+#define EFFECT_MEMORY__(p, m) do { \
+	if ((p) == 0) { (p) = (m); } else { (m) = (p); } \
+} while (0)
+
+#define EFFECT_MEMORY(p, m) do { \
+	if (HAS_QUIRK(QUIRK_S3MPMEM)) { \
+		EFFECT_MEMORY__((p), xc->vol.memory); \
+	} else { \
+		EFFECT_MEMORY__((p), (m)); \
+	} \
+} while (0)
+
+#define EFFECT_MEMORY_SETONLY(p, m) do { \
+	EFFECT_MEMORY__((p), (m)); \
+	if (HAS_QUIRK(QUIRK_S3MPMEM)) { \
+		if ((p) != 0) { xc->vol.memory = (p); } \
+	} \
+} while (0)
+
+#define EFFECT_MEMORY_S3M(p) do { \
+	if (HAS_QUIRK(QUIRK_S3MPMEM)) { \
+		EFFECT_MEMORY__((p), xc->vol.memory); \
+	} \
+} while (0)
+
 
 static void do_toneporta(struct module_data *m,
                                 struct channel_data *xc, int note)
@@ -49,7 +74,7 @@ static void do_toneporta(struct module_data *m,
 	if (note >= 1 && note <= 0x80 && (uint32)xc->ins < m->mod.ins) {
 		note--;
 		xc->porta.target = note_to_period(note + sub->xpo +
-			instrument->map[xc->key].xpo, sub->fin,
+			instrument->map[xc->key].xpo, xc->finetune,
 			HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
 	}
 	xc->porta.dir = xc->period < xc->porta.target ? 1 : -1;
@@ -76,11 +101,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		}
 		break;
 	case FX_S3M_ARPEGGIO:
-		if (fxp == 0) {
-			fxp = xc->arpeggio.memory;
-		} else {
-			xc->arpeggio.memory = fxp;
-		}
+		EFFECT_MEMORY(fxp, xc->arpeggio.memory);
 		goto fx_arpeggio;
 #ifndef LIBXMP_CORE_PLAYER
 	case FX_OKT_ARP3:
@@ -110,11 +131,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		break;
 #endif
 	case FX_PORTA_UP:	/* Portamento up */
-		if (fxp == 0) {
-			fxp = xc->freq.memory;
-		} else {
-			xc->freq.memory = fxp;
-		}
+		EFFECT_MEMORY(fxp, xc->freq.memory);
 
 		if (HAS_QUIRK(QUIRK_FINEFX)
 		    && (fnum == 0 || !HAS_QUIRK(QUIRK_ITVPOR))) {
@@ -140,11 +157,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		}
 		break;
 	case FX_PORTA_DN:	/* Portamento down */
-		if (fxp == 0) {
-			fxp = xc->freq.memory;
-		} else {
-			xc->freq.memory = fxp;
-		}
+		EFFECT_MEMORY(fxp, xc->freq.memory);
 
 		if (HAS_QUIRK(QUIRK_FINEFX)
 		    && (fnum == 0 || !HAS_QUIRK(QUIRK_ITVPOR))) {
@@ -179,27 +192,27 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 
 		do_toneporta(m, xc, note);
 
-		if (fxp) {
-			xc->porta.memory = fxp;
+		EFFECT_MEMORY_SETONLY(fxp, xc->porta.memory);
+
+		if (fxp != 0) {
 			if (HAS_QUIRK(QUIRK_UNISLD)) /* IT compatible Gxx off */
 				xc->freq.memory = fxp;
-		} else {
-			fxp = xc->porta.memory;
-		}
-
-		if (fxp) {
 			xc->porta.slide = fxp;
 		}
 		SET(TONEPORTA);
 		break;
 
 	case FX_VIBRATO:	/* Vibrato */
+		EFFECT_MEMORY_SETONLY(fxp, xc->vibrato.memory);
+
 		SET(VIBRATO);
-		SET_LFO_NOTZERO(&xc->vibrato, LSN(fxp) << 2, MSN(fxp));
+		SET_LFO_NOTZERO(&xc->vibrato.lfo, LSN(fxp) << 2, MSN(fxp));
 		break;
-	case FX_FINE_VIBRATO:	/* Fine vibrato (4x) */
+	case FX_FINE_VIBRATO:	/* Fine vibrato */
+		EFFECT_MEMORY_SETONLY(fxp, xc->vibrato.memory);
+
 		SET(VIBRATO);
-		SET_LFO_NOTZERO(&xc->vibrato, LSN(fxp), MSN(fxp));
+		SET_LFO_NOTZERO(&xc->vibrato.lfo, LSN(fxp), MSN(fxp));
 		break;
 
 	case FX_TONE_VSLIDE:	/* Toneporta + vol slide */
@@ -213,8 +226,9 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		goto fx_volslide;
 
 	case FX_TREMOLO:	/* Tremolo */
+		EFFECT_MEMORY_SETONLY(fxp, xc->tremolo.memory);
 		SET(TREMOLO);
-		SET_LFO_NOTZERO(&xc->tremolo, LSN(fxp), MSN(fxp));
+		SET_LFO_NOTZERO(&xc->tremolo.lfo, LSN(fxp), MSN(fxp));
 		break;
 
 	case FX_SETPAN:		/* Set pan */
@@ -247,11 +261,15 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 				xc->vol.memory = fxp;
 				fxp &= 0x0f;
 				goto fx_f_vslide_dn;
-			} else if (fxp == 0x00) {
-				if ((fxp = xc->vol.memory) != 0)
-					goto fx_volslide;
 			}
 		}
+
+		/* recover memory */
+		if (fxp == 0x00) {
+			if ((fxp = xc->vol.memory) != 0)
+				goto fx_volslide;
+		}
+
 		SET(VOL_SLIDE);
 
 		/* Skaven's 2nd reality (S3M) has volslide parameter D7 => pri
@@ -261,10 +279,11 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		 * of Sounds.xm
 		 */
 		if (fxp) {
+			xc->vol.memory = fxp;
 			h = MSN(fxp);
 			l = LSN(fxp);
 			if (HAS_QUIRK(QUIRK_VOLPDN)) {
-				if ((xc->vol.memory = fxp))
+				if (fxp)
 					xc->vol.slide = l ? -l : h;
 			} else {
 				if (l == 0)
@@ -304,6 +323,8 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	case FX_JUMP:		/* Order jump */
 		p->flow.pbreak = 1;
 		p->flow.jump = fxp;
+		/* effect B resets effect D in lower channels */
+		p->flow.jumpline = 0;
 		break;
 	case FX_VOLSET:		/* Volume set */
 		SET(NEW_VOL);
@@ -314,12 +335,15 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		p->flow.jumpline = 10 * MSN(fxp) + LSN(fxp);
 		break;
 	case FX_EXTENDED:	/* Extended effect */
+		EFFECT_MEMORY_S3M(fxp);
 		fxt = fxp >> 4;
 		fxp &= 0x0f;
 		switch (fxt) {
 		case EX_F_PORTA_UP:	/* Fine portamento up */
+		    EFFECT_MEMORY(fxp, xc->fine_porta.up_memory);
 		    goto fx_f_porta_up;
 		case EX_F_PORTA_DN:	/* Fine portamento down */
+		    EFFECT_MEMORY(fxp, xc->fine_porta.down_memory);
 		    goto fx_f_porta_dn;
 		case EX_GLISS:		/* Glissando toggle */
 			xc->gliss = fxp;
@@ -328,11 +352,13 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			fxp &= 3;
 			if (HAS_QUIRK(QUIRK_S3MLFO) && fxp == 2)
 				fxp |= 0x10;
-			set_lfo_waveform(&xc->vibrato, fxp);
+			set_lfo_waveform(&xc->vibrato.lfo, fxp);
 			break;
 		case EX_FINETUNE:	/* Set finetune */
 			fxp <<= 4;
-			goto fx_finetune;
+			if (!HAS_QUIRK(QUIRK_XMFINE) || note > 0)
+				goto fx_finetune;
+			break;
 		case EX_PATTERN_LOOP:	/* Loop pattern */
 			if (fxp == 0) {
 				/* mark start of loop */
@@ -357,7 +383,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			}
 			break;
 		case EX_TREMOLO_WF:	/* Set tremolo waveform */
-			set_lfo_waveform(&xc->tremolo, fxp & 3);
+			set_lfo_waveform(&xc->tremolo.lfo, fxp & 3);
 			break;
 		case EX_RETRIG:		/* Retrig note */
 			SET(RETRIG);
@@ -366,11 +392,14 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			xc->retrig.type = 0;
 			break;
 		case EX_F_VSLIDE_UP:	/* Fine volume slide up */
+			EFFECT_MEMORY(fxp, xc->fine_vol.up_memory);
 			goto fx_f_vslide_up;
 		case EX_F_VSLIDE_DN:	/* Fine volume slide down */
+			EFFECT_MEMORY(fxp, xc->fine_vol.down_memory);
 			goto fx_f_vslide_dn;
 		case EX_CUT:		/* Cut note */
 			SET(RETRIG);
+			SET_NOTE(NOTE_CUT);	/* for IT cut-carry */
 			xc->retrig.val = fxp + 1;
 			xc->retrig.count = xc->retrig.val;
 			xc->retrig.type = 0x10;
@@ -400,32 +429,30 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		break;
 
 	case FX_F_VSLIDE_UP:	/* Fine volume slide up */
+		EFFECT_MEMORY(fxp, xc->fine_vol.up_memory);
 	    fx_f_vslide_up:
 		SET(FINE_VOLS);
-		if (fxp)
-			xc->vol.fslide = fxp;
+		xc->vol.fslide = fxp;
 		break;
 	case FX_F_VSLIDE_DN:	/* Fine volume slide down */
+		EFFECT_MEMORY(fxp, xc->fine_vol.up_memory);
 	    fx_f_vslide_dn:
 		SET(FINE_VOLS);
-		if (fxp)
-			xc->vol.fslide = -fxp;
+		xc->vol.fslide = -fxp;
 		break;
 	case FX_F_PORTA_UP:	/* Fine portamento up */
 	    fx_f_porta_up:
-		SET(FINE_BEND);
-		if (fxp)
+		if (fxp) {
+			SET(FINE_BEND);
 			xc->freq.fslide = -fxp;
-		else if (xc->freq.slide > 0)
-			xc->freq.slide *= -1;
+		}
 		break;
 	case FX_F_PORTA_DN:	/* Fine portamento down */
 	    fx_f_porta_dn:
-		SET(FINE_BEND);
-		if (fxp)
+		if (fxp) {
+			SET(FINE_BEND);
 			xc->freq.fslide = fxp;
-		else if (xc->freq.slide < 0)
-			xc->freq.slide *= -1;
+		} 
 		break;
 	case FX_PATT_DELAY:
 	    fx_patt_delay:
@@ -433,13 +460,14 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		break;
 
 	case FX_S3M_SPEED:	/* Set S3M speed */
+		EFFECT_MEMORY_S3M(fxp);
 	    fx_s3m_speed:
 		if (fxp)
 			p->speed = fxp;
 		break;
 	case FX_S3M_BPM:	/* Set S3M BPM */
             fx_s3m_bpm:
-		if (fxp) {
+		if (fxp >= 0x20) {
 			if (fxp < XMP_MIN_BPM)
 				fxp = XMP_MIN_BPM;
 			p->bpm = fxp;
@@ -512,6 +540,8 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	case FX_ENVPOS:		/* Set envelope position */
 		/* FIXME: Add OpenMPT quirk */
 		xc->v_idx = fxp;
+		xc->f_idx = fxp;
+		xc->p_idx = fxp;
 		break;
 	case FX_PANSLIDE:	/* Pan slide (XM) */
 		/* TODO: add memory */
@@ -541,6 +571,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 #endif
 
 	case FX_MULTI_RETRIG:	/* Multi retrig */
+		EFFECT_MEMORY_S3M(fxp);
 		if (fxp) {
 			xc->retrig.val = fxp;
 		}
@@ -551,11 +582,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		SET(RETRIG);
 		break;
 	case FX_TREMOR:		/* Tremor */
-		if (fxp != 0) {
-			xc->tremor.memory = fxp;
-		} else {
-			fxp = xc->tremor.memory;
-		}
+		EFFECT_MEMORY(fxp, xc->tremor.memory);
 		if (MSN(fxp) == 0)
 			fxp |= 0x10;
 		if (LSN(fxp) == 0)
@@ -623,6 +650,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			xc->trackvol.fslide = MSN(fxp) - LSN(fxp);
 		}
 		break;
+
 	case FX_IT_INSTFUNC:
 		switch (fxp) {
 		case 0:	/* Past note cut */
@@ -660,10 +688,10 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		break;
 	case FX_PANBRELLO:	/* Panbrello */
 		SET(PANBRELLO);
-		SET_LFO_NOTZERO(&xc->panbrello, LSN(fxp) << 4, MSN(fxp));
+		SET_LFO_NOTZERO(&xc->panbrello.lfo, LSN(fxp) << 4, MSN(fxp));
 		break;
 	case FX_PANBRELLO_WF:	/* Panbrello waveform */
-		set_lfo_waveform(&xc->panbrello, fxp & 3);
+		set_lfo_waveform(&xc->panbrello.lfo, fxp & 3);
 		break;
 #endif
 
@@ -756,7 +784,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		} else {
 			RESET_PER(VIBRATO);
 		}
-		SET_LFO_NOTZERO(&xc->vibrato, LSN(fxp) << 2, MSN(fxp));
+		SET_LFO_NOTZERO(&xc->vibrato.lfo, LSN(fxp) << 2, MSN(fxp));
 		break;
 	case FX_PER_PORTA_UP:	/* Persistent portamento up */
 		SET_PER(PITCHBEND);
@@ -793,7 +821,7 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		break;
 	case FX_VIBRATO2:	/* Deep vibrato (2x) */
 		SET(VIBRATO);
-		SET_LFO_NOTZERO(&xc->vibrato, LSN(fxp) << 3, MSN(fxp));
+		SET_LFO_NOTZERO(&xc->vibrato.lfo, LSN(fxp) << 3, MSN(fxp));
 		break;
 	case FX_SPEED_CP:	/* Set speed and ... */
 		if (fxp)
