@@ -72,6 +72,7 @@ extern const struct format_loader mfp_loader;
 extern const struct format_loader pw_loader;
 extern const struct format_loader hmn_loader;
 extern const struct format_loader chip_loader;
+extern const struct format_loader abk_loader;
 
 extern const struct pw_format *const pw_format[];
 
@@ -133,6 +134,7 @@ const struct format_loader *const format_loader[NUM_FORMATS + 2] = {
 	&rad_loader,
 	&hsc_loader,
 	&mfp_loader,
+	&abk_loader,
 	/* &alm_loader, */
 	/* &polly_loader, */
 	/* &stc_loader, */
