@@ -4,6 +4,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stddef.h>
+#include "memio.h"
 
 #define HIO_HANDLE_TYPE(x) ((x)->type)
 
@@ -11,10 +12,10 @@ typedef struct {
 #define HIO_HANDLE_TYPE_FILE	0
 #define HIO_HANDLE_TYPE_MEMORY	1
 	int type;
-	FILE *f;
-	uint8 *start;
-	ptrdiff_t pos;
-	ptrdiff_t size;
+	union {
+		FILE *file;
+		MFILE *mem;
+	} handle;
 } HIO_HANDLE;
 
 int8	hio_read8s	(HIO_HANDLE *);
@@ -29,9 +30,10 @@ size_t	hio_read	(void *, size_t, size_t, HIO_HANDLE *);
 int	hio_seek	(HIO_HANDLE *, long, int);
 long	hio_tell	(HIO_HANDLE *);
 int	hio_eof		(HIO_HANDLE *);
-HIO_HANDLE *hio_open_file (void *, char *);
+HIO_HANDLE *hio_open	(void *, char *);
 HIO_HANDLE *hio_open_mem  (void *, long);
 HIO_HANDLE *hio_open_fd	  (int, char *);
+HIO_HANDLE *hio_open_file (FILE *);
 int	hio_close	(HIO_HANDLE *);
 int	hio_stat	(HIO_HANDLE *, struct stat *);
 
