@@ -446,11 +446,11 @@ int xmp_test_module(char *path, struct xmp_test_info *info)
 	}
 #endif
 
-	if ((h = hio_open_file(path, "rb")) == NULL)
+	if ((h = hio_open(path, "rb")) == NULL)
 		return -XMP_ERROR_SYSTEM;
 
 #ifndef LIBXMP_CORE_PLAYER
-	if (decrunch(&h->f, path, &temp) < 0) {
+	if (decrunch(&h->handle.file, path, &temp) < 0) {
 		ret = -XMP_ERROR_DEPACK;
 		goto err;
 	}
@@ -472,19 +472,19 @@ int xmp_test_module(char *path, struct xmp_test_info *info)
 	}
 
 	for (i = 0; format_loader[i] != NULL; i++) {
-		fseek(h->f, 0, SEEK_SET);
+		fseek(h->handle.file, 0, SEEK_SET);
 		if (format_loader[i]->test(h, buf, 0) == 0) {
 			int is_prowizard = 0;
 
 #ifndef LIBXMP_CORE_PLAYER
 			if (strcmp(format_loader[i]->name, "prowizard") == 0) {
-				fseek(h->f, 0, SEEK_SET);
-				pw_test_format(h->f, buf, 0, info);
+				fseek(h->handle.file, 0, SEEK_SET);
+				pw_test_format(h->handle.file, buf, 0, info);
 				is_prowizard = 1;
 			}
 #endif
 
-			fclose(h->f);
+			fclose(h->handle.file);
 
 #ifndef LIBXMP_CORE_PLAYER
 			unlink_tempfile(temp);
@@ -585,12 +585,12 @@ int xmp_load_module(xmp_context opaque, char *path)
 	}
 #endif
 
-	if ((h = hio_open_file(path, "rb")) == NULL)
+	if ((h = hio_open(path, "rb")) == NULL)
 		return -XMP_ERROR_SYSTEM;
 
 #ifndef LIBXMP_CORE_PLAYER
 	D_(D_INFO "decrunch");
-	if (decrunch(&h->f, path, &temp) < 0)
+	if (decrunch(&h->handle.file, path, &temp) < 0)
 		goto err_depack;
 
 	if (hio_stat(h, &st) < 0)
@@ -653,6 +653,31 @@ int xmp_load_module_from_memory(xmp_context opaque, void *mem, long size)
 	return load_module(opaque, h, NULL);
 }
 
+int xmp_load_module_from_file(xmp_context opaque, void *file, long size)
+{
+	struct context_data *ctx = (struct context_data *)opaque;
+	struct module_data *m = &ctx->m;
+	HIO_HANDLE *h;
+	FILE *f = (FILE *)file;
+	struct stat st;
+
+	if (fstat(fileno(f), &st) < 0)
+		return -XMP_ERROR_SYSTEM;
+
+	if ((h = hio_open_file(f)) == NULL)
+		return -XMP_ERROR_SYSTEM;
+
+	if (ctx->state > XMP_STATE_UNLOADED)
+		xmp_release_module(opaque);
+
+	m->filename = NULL;
+	m->basename = NULL;
+	m->dirname = NULL;
+	m->size = st.st_size;
+
+	return load_module(opaque, h, NULL);
+}
+
 void xmp_release_module(xmp_context opaque)
 {
 	struct context_data *ctx = (struct context_data *)opaque;
