@@ -82,13 +82,14 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	int ver = 0;
 	int smp_idx = 0;
 	uint8 e[4];
-	uint32 song_offset;
-	uint32 blockarr_offset;
-	uint32 smplarr_offset;
-	uint32 expdata_offset;
-	uint32 expsmp_offset;
-	uint32 songname_offset;
-	uint32 iinfo_offset;
+	int song_offset;
+	int blockarr_offset;
+	int smplarr_offset;
+	int expdata_offset;
+	int expsmp_offset;
+	int songname_offset;
+	int iinfo_offset;
+	int annotxt_offset;
 	int pos;
 	int bpm_on, bpmlen, med_8ch, hexvol;
 
@@ -218,8 +219,8 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		D_(D_INFO "expsmp_offset = 0x%08x", expsmp_offset);
 		expdata.s_ext_entries = hio_read16b(f);
 		expdata.s_ext_entrsz = hio_read16b(f);
-		hio_read32b(f);
-		hio_read32b(f);
+		annotxt_offset = hio_read32b(f);
+		expdata.annolen = hio_read32b(f);
 		iinfo_offset = hio_read32b(f);
 		D_(D_INFO "iinfo_offset = 0x%08x", iinfo_offset);
 		expdata.i_ext_entries = hio_read16b(f);
@@ -229,15 +230,24 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		hio_read32b(f);
 		hio_read32b(f);
 		songname_offset = hio_read32b(f);
-		D_(D_INFO "songname_offset = 0x%08x", songname_offset);
 		expdata.songnamelen = hio_read32b(f);
+
 		hio_seek(f, start + songname_offset, SEEK_SET);
-		D_(D_INFO "expdata.songnamelen = %d", expdata.songnamelen);
 		for (i = 0; i < expdata.songnamelen; i++) {
 			if (i >= XMP_NAME_SIZE)
 				break;
 			mod->name[i] = hio_read8(f);
 		}
+
+		/* Read annotation */
+		if (annotxt_offset != 0 && expdata.annolen != 0) {
+			m->comment = malloc(expdata.annolen + 1);
+			if (m->comment != NULL) {
+				hio_seek(f, start + annotxt_offset, SEEK_SET);
+				hio_read(m->comment, 1, expdata.annolen, f);
+				m->comment[expdata.annolen] = 0;
+			}
+		}
 	}
 
 	/*
@@ -324,8 +334,7 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 						event->note +=
 						    12 + song.playtransp;
 
-					if (event->note < 0 || event->note >=
-								XMP_MAX_KEYS)
+					if (event->note >= XMP_MAX_KEYS)
 						event->note = 0;
 
 					event->ins = e[1] & 0x3f;
@@ -353,8 +362,7 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 					if (event->note)
 						event->note += 12;
 
-					if (event->note < 0 || event->note >=
-								XMP_MAX_KEYS)
+					if (event->note >= XMP_MAX_KEYS)
 						event->note = 0;
 
 					event->ins =
@@ -389,7 +397,6 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	for (smp_idx = i = 0; i < mod->ins; i++) {
 		int smpl_offset;
-		char name[40] = "";
 
 		if (hio_seek(f, start + smplarr_offset + i * 4, SEEK_SET) != 0)
 			return -1;
@@ -401,16 +408,17 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			continue;
 
 		if (hio_seek(f, start + smpl_offset, SEEK_SET) != 0)
-		  return -1;
+			return -1;
 		instr.length = hio_read32b(f);
 		instr.type = hio_read16b(f);
 
 		pos = hio_tell(f);
 
 		if (expdata_offset && i < expdata.i_ext_entries) {
-		    if (hio_seek(f, iinfo_offset + i * expdata.i_ext_entrsz, SEEK_SET) != 0)
-		      return -1;
-		    hio_read(name, 40, 1, f);
+			struct xmp_instrument *xxi = &mod->xxi[i];
+			if (hio_seek(f, iinfo_offset + i * expdata.i_ext_entrsz, SEEK_SET) != 0)
+				return -1;
+			hio_read(&xxi->name, 40, 1, f);
 		}
 
 		D_(D_INFO "\n[%2x] %-40.40s %d", i, name, instr.type);
@@ -497,7 +505,7 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	for (i = 0; i < mod->chn; i++) {
 		mod->xxc[i].vol = song.trkvol[i];
-		mod->xxc[i].pan = (((i + 1) / 2) % 2) * 0xff;
+		mod->xxc[i].pan = DEFPAN((((i + 1) / 2) % 2) * 0xff);
 	}
 
 	m->read_event_type = READ_EVENT_MED;
