@@ -203,9 +203,9 @@ void mmd_xlat_fx(struct xmp_event *event, int bpm_on, int bpmlen, int med_8ch,
 		case 0xfe:	/* End of song */
 			event->fxt = event->fxp = 0;
 			break;
-		case 0xff:	/* Note cut */
-			event->fxt = FX_EXTENDED;
-			event->fxp = (EX_CUT << 4) | 3;
+		case 0xff:	/* Turn note off */
+			event->fxt = event->fxp = 0;
+			event->note = XMP_KEY_CUT;
 			break;
 		default:
 			event->fxt = event->fxp = 0;
@@ -235,10 +235,13 @@ void mmd_xlat_fx(struct xmp_event *event, int bpm_on, int bpmlen, int med_8ch,
 	case 0x15:
 		/* SET FINETUNE 15
 		 * Set a finetune value for a note, overrides the default
-		 * fine tune value of the instrument.
+		 * fine tune value of the instrument. Negative numbers must
+		 * be entered as follows:
+		 *   -1 => FF    -3 => FD    -5 => FB    -7 => F9
+		 *   -2 => FE    -4 => FC    -6 => FA    -8 => F8
 		 */
-		/* FIXME */
 		event->fxt = FX_FINETUNE;
+		event->fxp = (event->fxp + 8) << 4;
 		break;
 	case 0x16:
 		/* LOOP 16
@@ -520,12 +523,9 @@ int mmd_load_sampled_instrument(HIO_HANDLE *f, struct module_data *m, int i,
 	sub->pan = 0x80;
 	sub->xpo = sample->strans + 36;
 	if (ver >= 2 && expdata->s_ext_entrsz > 4) {	/* MMD2+ */
-		sub->xpo += exp_smp->default_pitch;
-#if 0
-		if (ver == 2) {
-			sub->xpo += 24;			/* ??!? */
+		if (exp_smp->default_pitch) {
+			sub->xpo += exp_smp->default_pitch - 25;
 		}
-#endif
 	}
 	sub->sid = smp_idx;
 	sub->fin = exp_smp->finetune << 4;
@@ -697,3 +697,23 @@ void mmd_set_bpm(struct module_data *m, int med_8ch, int deftempo,
 	}
 }
 
+
+void mmd_info_text(HIO_HANDLE *f, struct module_data *m, int offset)
+{
+	int type, len;
+
+	/* Copy song info text */
+	hio_read32b(f);		/* skip next */
+	hio_read16b(f);		/* skip reserved */
+	type = hio_read16b(f);
+	if (type == 1) {	/* 1 = ASCII */
+		len = hio_read32b(f);
+		if (len != 0) {
+			m->comment = malloc(len + 1);
+			if (m->comment == NULL)
+				return;
+			hio_read(m->comment, 1, len, f);
+			m->comment[len] = 0;
+		}
+	}
+}
