@@ -67,6 +67,8 @@ struct mod_magic {
 #define TRACKER_CLONE		98
 #define TRACKER_UNKNOWN		99
 
+#define TRACKER_PROBABLY_NOISETRACKER 20
+
 const struct mod_magic mod_magic[] = {
     { "M.K.", 0, TRACKER_PROTRACKER, 4 },
     { "M!K!", 1, TRACKER_PROTRACKER, 4 },
@@ -132,12 +134,12 @@ static int mod_test(HIO_HANDLE *f, char *t, const int start)
 
     hio_seek(f, start + 20, SEEK_SET);
     for (i = 0; i < 31; i++) {
-	hio_seek(f, 22, SEEK_CUR);			/* Instrument name */
+	hio_seek(f, 22, SEEK_CUR);		/* Instrument name */
 	if (hio_read16b(f) & 0x8000)		/* test length */
 		return -1;
-	if (hio_read8(f) & 0xf0)			/* test finetune */
+	if (hio_read8(f) & 0xf0)		/* test finetune */
 		return -1;
-	if (hio_read8(f) > 0x40)			/* test volume */
+	if (hio_read8(f) > 0x40)		/* test volume */
 		return -1;
 	if (hio_read16b(f) & 0x8000)		/* test loop start */
 		return -1;
@@ -220,7 +222,6 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
     char magic[8], idbuffer[32];
     int ptkloop = 0;			/* Protracker loop */
     int tracker_id = TRACKER_PROTRACKER;
-    int showmagic;
 
     LOAD_INIT();
 
@@ -269,7 +270,6 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	}
 	tracker_id = mod->chn & 1 ? TRACKER_TAKETRACKER : TRACKER_FASTTRACKER2;
 	detected = 1;
-	m->quirk &= ~QUIRK_MODRNG;
     }
 
     strncpy(mod->name, (char *) mh.name, 20);
@@ -374,7 +374,9 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
     if (mod->chn == 4 && mh.restart == mod->pat) {
 	tracker_id = TRACKER_SOUNDTRACKER;
     } else if (mod->chn == 4 && mh.restart == 0x78) {
-        tracker_id = TRACKER_NOISETRACKER;
+	/* Can't trust this for Noisetracker, MOD.Data City Remix has
+	 * Protracker effects and Noisetracker restart byte */
+        tracker_id = TRACKER_PROBABLY_NOISETRACKER;
     } else if (mh.restart < 0x7f) {
 	if (mod->chn == 4) {
 	    tracker_id = TRACKER_NOISETRACKER;
@@ -386,7 +388,6 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
     if (mod->chn != 4 && mh.restart == 0x7f) {
 	tracker_id = TRACKER_SCREAMTRACKER3;
-	m->quirk &= ~QUIRK_MODRNG;
 	m->read_event_type = READ_EVENT_ST3;
     }
 
@@ -439,7 +440,6 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	    	    tracker_id = TRACKER_PROTRACKER;
 		} else if (mod->chn == 6 || mod->chn == 8) {
 	    	    tracker_id = TRACKER_FASTTRACKER;	/* FastTracker 1.01? */
-		    m->quirk &= ~QUIRK_MODRNG;
 		} else {
 	    	    tracker_id = TRACKER_UNKNOWN;
 		}
@@ -466,7 +466,6 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	    if (mod->chn == 4 || mod->chn == 6 || mod->chn == 8) {
 	    	tracker_id = TRACKER_FASTTRACKER;
-	        m->quirk &= ~QUIRK_MODRNG;
 		goto skip_test;
 	    }
 
@@ -476,70 +475,8 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 skip_test:
 
-    showmagic = 0;
-
-    switch (tracker_id) {
-    case TRACKER_PROTRACKER:
-	tracker = "Protracker";
-	ptkloop = 1;
-	break;
-    case TRACKER_NOISETRACKER:
-	tracker = "Noisetracker";
-	ptkloop = 1;
-	break;
-    case TRACKER_SOUNDTRACKER:
-	tracker = "Soundtracker";
-	ptkloop = 1;
-	break;
-    case TRACKER_FASTTRACKER:
-    case TRACKER_FASTTRACKER2:
-    case TRACKER_TAKETRACKER:
-	tracker = "Fast Tracker";
-	break;
-    case TRACKER_OCTALYSER:
-	tracker = "Octalyser";
-	break;
-    case TRACKER_DIGITALTRACKER:
-	tracker = "Digital Tracker";
-	break;
-    case TRACKER_FLEXTRAX:
-	tracker = "Flextrax";
-	break;
-    case TRACKER_MODSGRAVE:
-	tracker = "Mod's Grave";
-	break;
-    case TRACKER_SCREAMTRACKER3:
-	tracker = "Scream Tracker";
-	showmagic = 1;
-	break;
-	break;
-    case TRACKER_CONVERTEDST:
-    case TRACKER_CONVERTED:
-	tracker = "Converted";
-	showmagic = 1;
-	break;
-    case TRACKER_CLONE:
-	tracker = "Protracker clone";
-	break;
-    default:
-    case TRACKER_UNKNOWN_CONV:
-    case TRACKER_UNKNOWN:
-	tracker = "Unknown tracker";
-	showmagic = 1;
-	break;
-    }
-
     mod->trk = mod->chn * mod->pat;
 
-    if ((!memcmp(magic, "M.K.", 4) && showmagic == 0)
-			|| tracker_id == TRACKER_FLEXTRAX) {
-        snprintf(mod->type, XMP_NAME_SIZE, "%s", tracker);
-    } else {
-        snprintf(mod->type, XMP_NAME_SIZE, "%s %s", tracker, magic);
-    }
-
-    MODULE_INFO();
-
     for (i = 0; i < mod->ins; i++) {
 	D_(D_INFO "[%2X] %-22.22s %04x %04x %04x %c V%02x %+d %c\n",
 		i, mod->xxi[i].name,
@@ -565,18 +502,90 @@ skip_test:
 	    event = &EVENT (i, j % mod->chn, j / mod->chn);
 	    hio_read (mod_event, 1, 4, f);
 
+	    /* Filter noisetracker events */
+	    if (tracker_id == TRACKER_PROBABLY_NOISETRACKER) {
+		unsigned char fxt = LSN(mod_event[2]);
+		unsigned char fxp = LSN(mod_event[3]);
+        	if ((fxt > 0x06 && fxt < 0x0a) ||  (fxt == 0x0e && fxp > 1)) {
+		    tracker_id = TRACKER_UNKNOWN;
+		}
+	    }
+
 	    switch (tracker_id) {
+	    case TRACKER_PROBABLY_NOISETRACKER:
 	    case TRACKER_NOISETRACKER:
-		decode_noisetracker_event(event, mod_event);
-		break;
-	    case TRACKER_PROTRACKER:
-	    default:
-		decode_protracker_event(event, mod_event);
+	    	decode_noisetracker_event(event, mod_event);
 		break;
+	    default:	
+	        decode_protracker_event(event, mod_event);
 	    }
 	}
     }
 
+    switch (tracker_id) {
+    case TRACKER_PROTRACKER:
+	tracker = "Protracker";
+	ptkloop = 1;
+	break;
+    case TRACKER_PROBABLY_NOISETRACKER:
+    case TRACKER_NOISETRACKER:
+	tracker = "Noisetracker";
+	ptkloop = 1;
+	break;
+    case TRACKER_SOUNDTRACKER:
+	tracker = "Soundtracker";
+	ptkloop = 1;
+	break;
+    case TRACKER_FASTTRACKER:
+    case TRACKER_FASTTRACKER2:
+	tracker = "Fast Tracker";
+	m->quirk &= ~QUIRK_MODRNG;
+	break;
+    case TRACKER_TAKETRACKER:
+	tracker = "Take Tracker";
+	m->quirk &= ~QUIRK_MODRNG;
+	break;
+    case TRACKER_OCTALYSER:
+	tracker = "Octalyser";
+	break;
+    case TRACKER_DIGITALTRACKER:
+	tracker = "Digital Tracker";
+	break;
+    case TRACKER_FLEXTRAX:
+	tracker = "Flextrax";
+	break;
+    case TRACKER_MODSGRAVE:
+	tracker = "Mod's Grave";
+	break;
+    case TRACKER_SCREAMTRACKER3:
+	tracker = "Scream Tracker";
+	m->quirk &= ~QUIRK_MODRNG;
+	break;
+    case TRACKER_CONVERTEDST:
+    case TRACKER_CONVERTED:
+	tracker = "Converted";
+	break;
+    case TRACKER_CLONE:
+	tracker = "Protracker clone";
+	m->quirk &= ~QUIRK_MODRNG;
+	break;
+    default:
+    case TRACKER_UNKNOWN_CONV:
+    case TRACKER_UNKNOWN:
+	tracker = "Unknown tracker";
+	m->quirk &= ~QUIRK_MODRNG;
+	break;
+    }
+
+    if (tracker_id == TRACKER_MODSGRAVE) {
+	snprintf(mod->type, XMP_NAME_SIZE, "%s", tracker);
+    } else {
+	snprintf(mod->type, XMP_NAME_SIZE, "%s %s", tracker, magic);
+    }
+
+    MODULE_INFO();
+
+
     /* Load samples */
 
     if (m->filename && (x = strrchr(m->filename, '/')))
@@ -597,7 +606,7 @@ skip_test:
 	    char sn[256];
 	    snprintf(sn, XMP_NAME_SIZE, "%s%s", pathname, mod->xxi[i].name);
 	
-	    if ((s = hio_open_file(sn, "rb"))) {
+	    if ((s = hio_open(sn, "rb"))) {
 	        if (load_sample(m, s, flags, &mod->xxs[i], NULL) < 0) {
 		    hio_close(s);
 		    return -1;
