@@ -102,7 +102,7 @@ static int psm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		hio_read(buf, 1, 13, f);	/* sample filename */
 		hio_read(buf, 1, 24, f);	/* sample description */
 		strncpy((char *)mod->xxi[i].name, (char *)buf, 24);
-		str_adj((char *)mod->xxi[i].name);
+		adjust_string((char *)mod->xxi[i].name);
 		p_smp[i] = hio_read32l(f);
 		hio_read32l(f);			/* memory location */
 		hio_read16l(f);			/* sample number */
