@@ -80,10 +80,13 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
      * Mon, 04 Jan 1999 11:17:20 +0100
      */
     for (i = 0; i < mod->pat - 1; i++) {
+    	const int headsize = version > 0x0102 ? 9 : 8;
+
 	xph.length = hio_read32l(f);
 	xph.packing = hio_read8(f);
 	xph.rows = version > 0x0102 ? hio_read16l(f) : hio_read8(f) + 1;
 	xph.datasize = hio_read16l(f);
+	hio_seek(f, xph.length - headsize, SEEK_CUR);
 
 	r = xph.rows;
 	if (r == 0)
@@ -169,7 +172,7 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 		    break;
 		case 0x0c:	/* Set panning */
 		    event->f2t = FX_SETPAN;
-		    event->f2p = ((event->vol - 0xc0) << 4) + 8;
+		    event->f2p = (event->vol - 0xc0) << 4;
 		    break;
 		case 0x0d:	/* Pan slide left */
 		    event->f2t = FX_PANSLIDE;
