@@ -116,11 +116,12 @@ int med_linear_bend(struct context_data *ctx, struct channel_data *xc)
 }
 
 
-void med_play_extras(struct context_data *ctx, struct channel_data *xc, int chn,
-		     int new_note)
+void med_play_extras(struct context_data *ctx, struct channel_data *xc,
+			int chn, int t)
 {
 	struct module_data *m = &ctx->m;
 	struct player_data *p = &ctx->p;
+	struct xmp_module *mod = &m->mod;
 	struct med_module_extras *me;
 	struct med_channel_extras *ce;
 	struct med_instrument_extras *ie;
@@ -167,14 +168,20 @@ void med_play_extras(struct context_data *ctx, struct channel_data *xc, int chn,
 		return;
 	}
 
-	if (new_note) {
-		ce->arp = ce->aidx = 0;
+	if (t == 0 && TEST(NEW_NOTE)) {
 		ce->period = xc->period;
-		ce->vp = ce->vc = ce->vw = 0;
-		ce->wp = ce->wc = ce->ww = 0;
-		ce->vv = 0;
-		ce->vs = ie->vts;
-		ce->ws = ie->wts;
+		if (TEST(NEW_INS)) {
+			ce->arp = ce->aidx = 0;
+			ce->vp = ce->vc = ce->vw = 0;
+			ce->wp = ce->wc = ce->ww = 0;
+			ce->env_wav = -1;
+			ce->env_idx = 0;
+			ce->flags &= ~MED_SYNTH_ENV_LOOP;
+			ce->vv = 0;
+			ce->wv = 0;
+			ce->vs = ie->vts;
+			ce->ws = ie->wts;
+		}
 	}
 
 	if (ce->vs > 0 && ce->vc-- == 0) {
@@ -187,6 +194,8 @@ void med_play_extras(struct context_data *ctx, struct channel_data *xc, int chn,
 
 		loop = jws = 0;
 
+		/* Volume commands */
+
 	    next_vt:
 		switch (b = VT) {
 		case 0xff:	/* END */
@@ -205,8 +214,11 @@ void med_play_extras(struct context_data *ctx, struct channel_data *xc, int chn,
 			jws = VT;
 			break;
 		case 0xf5:	/* EN2 */
+			ce->env_wav = VT;
+			ce->flags |= MED_SYNTH_ENV_LOOP;
+			break;
 		case 0xf4:	/* EN1 */
-			VT_SKIP;	/* Not implemented */
+			ce->env_wav = VT;
 			break;
 		case 0xf3:	/* CHU */
 			ce->vv = VT;
@@ -225,11 +237,28 @@ void med_play_extras(struct context_data *ctx, struct channel_data *xc, int chn,
 				ce->volume = b;
 		}
 
+	    skip_vol:
+
+		/* volume envelope */
+		if (ce->env_wav >= 0) {
+			int sid = mod->xxi[xc->ins].sub[ce->env_wav].sid;
+			struct xmp_sample *xxs = &mod->xxs[sid];
+			if (xxs->len == 0x80) {		/* sanity check */
+				ce->volume = ((int8)xxs->data[ce->env_idx] + 0x80) >> 2;
+				ce->env_idx++;
+
+				if (ce->env_idx >= 0x80) {
+					if (~ce->flags & MED_SYNTH_ENV_LOOP) {
+						ce->env_wav = -1;
+					}
+					ce->env_idx = 0;
+				}
+			}
+		}
+
 		ce->volume += ce->vv;
 		CLAMP(ce->volume, 0, 64);
 
-	    skip_vol:
-
 		if (ce->ww > 0) {
 			ce->ww--;
 			goto skip_wav;
@@ -237,6 +266,8 @@ void med_play_extras(struct context_data *ctx, struct channel_data *xc, int chn,
 
 		loop = jvs = 0;
 
+		/* Waveform commands */
+
 	    next_wt:
 		switch (b = WT) {
 			struct xmp_instrument *xxi;
@@ -296,11 +327,10 @@ void med_play_extras(struct context_data *ctx, struct channel_data *xc, int chn,
 				virt_setsmp(ctx, chn, xc->smp);
 			}
 		}
-		xc->period += ce->wv;
 
 	    skip_wav:
-		;
-		/* xc->period += ce->wv; */
+
+		xc->period += ce->wv;
 	}
 
 	if (jws) {
