@@ -287,22 +287,6 @@ static void process_volume(struct context_data *ctx, int chn, int t, int act)
 			xc->fadeout = 0;
 			SET_NOTE(NOTE_END);
 		}
-
-		if (xc->fadeout == 0) {
-			/* Setting volume to 0 instead of resetting the channel
-			 * will use more CPU but allows portamento after keyoff
-			 * to continue the sample instead of resetting it.
-			 * This is used in Decibelter - Cosmic 'Wegian Mamas.xm
-			 * Only reset the channel in virtual channel mode so we
-			 * can release it.
-			 */
-			if (HAS_QUIRK(QUIRK_VIRTUAL)) {
-				virt_resetchannel(ctx, chn);
-				return;
-			} else {
-				xc->volume = 0;
-			}
-		}
 	}
 
 	switch (check_envelope_fade(&instrument->aei, xc->v_idx)) {
@@ -321,9 +305,13 @@ static void process_volume(struct context_data *ctx, int chn, int t, int act)
 	}
 
 	vol_envelope = get_envelope(&instrument->aei, xc->v_idx, 64, &end);
-	xc->v_idx = update_envelope(&instrument->aei, xc->v_idx, DOENV_RELEASE);
-	if (end && vol_envelope == 0)
-		SET_NOTE(NOTE_END);
+	xc->v_idx = update_envelope(&instrument->aei, xc->v_idx, DOENV_RELEASE,
+						HAS_QUIRK(QUIRK_ENVSUS));
+	if (end) {
+		if (vol_envelope == 0)
+			SET_NOTE(NOTE_END);
+		SET_NOTE(NOTE_ENV_END);
+	}
 
 	/* If note ended in background channel, we can safely reset it */
 	if (TEST_NOTE(NOTE_END) && chn >= p->virt.num_tracks) {
@@ -338,8 +326,8 @@ static void process_volume(struct context_data *ctx, int chn, int t, int act)
 #endif
 
 	if (TEST(TREMOLO)) {
-		finalvol += get_lfo(&xc->tremolo, 1 << 6);
-		update_lfo(&xc->tremolo);
+		finalvol += get_lfo(&xc->tremolo.lfo, 1 << 6);
+		update_lfo(&xc->tremolo.lfo);
 	}
 
 	CLAMP(finalvol, 0, m->volbase);
@@ -418,7 +406,8 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 	instrument = get_instrument(ctx, xc->ins);
 
 	frq_envelope = get_envelope(&instrument->fei, xc->f_idx, 0, &end);
-	xc->f_idx = update_envelope(&instrument->fei, xc->f_idx, DOENV_RELEASE);
+	xc->f_idx = update_envelope(&instrument->fei, xc->f_idx, DOENV_RELEASE,
+						HAS_QUIRK(QUIRK_ENVSUS));
 
 #ifndef LIBXMP_CORE_PLAYER
 	/* Do note slide */
@@ -446,7 +435,7 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 	/* Vibrato */
 	if (TEST(VIBRATO) || TEST_PER(VIBRATO)) {
 		int shift = HAS_QUIRK(QUIRK_VIBHALF) ? 10 : 9;
-		int vib = get_lfo(&xc->vibrato, 1 << shift);
+		int vib = get_lfo(&xc->vibrato.lfo, 1 << shift);
 
 		if (HAS_QUIRK(QUIRK_VIBINV)) {
 			vibrato -= vib;
@@ -455,7 +444,7 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 		}
 
 		if (t % p->speed != 0 || HAS_QUIRK(QUIRK_VIBALL)) {
-			update_lfo(&xc->vibrato);
+			update_lfo(&xc->vibrato.lfo);
 		}
 	}
 
@@ -464,8 +453,7 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 	period += extras_get_period(ctx, xc);
 #endif
 
-	linear_bend = period_to_bend(period + vibrato, xc->note,
-				HAS_QUIRK(QUIRK_MODRNG), xc->gliss,
+	linear_bend = period_to_bend(period + vibrato, xc->note, xc->gliss,
 				HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
 
 	/* Envelope */
@@ -492,6 +480,14 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 	xc->info_pitchbend = linear_bend >> 7;
 	xc->info_period = note_to_period_mix(xc->note, linear_bend);
 
+	if (HAS_QUIRK(QUIRK_MODRNG)) {
+		CLAMP(xc->info_period,
+			note_to_period(83, xc->finetune, 0, 0) * 4096,
+			note_to_period(48, xc->finetune, 0, 0) * 4096);
+	} else if (xc->info_period <  (1 << 12)) {
+		xc->info_period = (1 << 12);
+	}
+
 	virt_setbend(ctx, chn, linear_bend);
 
 #ifndef LIBXMP_CORE_DISABLE_IT
@@ -529,6 +525,7 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 static void process_pan(struct context_data *ctx, int chn, int t, int act)
 {
 	struct player_data *p = &ctx->p;
+	struct module_data *m = &ctx->m;
 	struct mixer_data *s = &ctx->s;
 	struct channel_data *xc = &p->xc_data[chn];
 	struct xmp_instrument *instrument;
@@ -539,11 +536,12 @@ static void process_pan(struct context_data *ctx, int chn, int t, int act)
 	instrument = get_instrument(ctx, xc->ins);
 
 	pan_envelope = get_envelope(&instrument->pei, xc->p_idx, 32, &end);
-	xc->p_idx = update_envelope(&instrument->pei, xc->p_idx, DOENV_RELEASE);
+	xc->p_idx = update_envelope(&instrument->pei, xc->p_idx, DOENV_RELEASE,
+						HAS_QUIRK(QUIRK_ENVSUS));
 
 	if (TEST(PANBRELLO)) {
-		panbrello = get_lfo(&xc->panbrello, 512);
-		update_lfo(&xc->panbrello);
+		panbrello = get_lfo(&xc->panbrello.lfo, 512);
+		update_lfo(&xc->panbrello.lfo);
 	}
 
 	finalpan = xc->pan.val + panbrello + (pan_envelope - 32) *
@@ -662,11 +660,18 @@ static void update_frequency(struct context_data *ctx, int chn, int t)
 		if (TEST(FINE_NSLIDE)) {
 			xc->note += xc->noteslide.fslide;
 			xc->period = note_to_period(xc->note, xc->finetune,
-					 HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+					HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
 		}
 #endif
 	}
 
+	if (HAS_QUIRK(QUIRK_LINEAR)) {
+		CLAMP(xc->period, MIN_PERIOD_L, MAX_PERIOD_L);
+	} else if (HAS_QUIRK(QUIRK_MODRNG)) {
+		CLAMP(xc->period, note_to_period(83, xc->finetune, 0, 0),
+				  note_to_period(48, xc->finetune, 0, 0));
+	}
+
 	/* Check for invalid periods (from Toru Egashira's NSPmod)
 	 * panic.s3m has negative periods
 	 * ambio.it uses low (~8) period values
@@ -675,12 +680,6 @@ static void update_frequency(struct context_data *ctx, int chn, int t)
 		xc->volume = 0;
 	}
 
-	if (HAS_QUIRK(QUIRK_LINEAR)) {
-		CLAMP(xc->period, MIN_PERIOD_L, MAX_PERIOD_L);
-	} else {
-		CLAMP(xc->period, MIN_PERIOD_A, MAX_PERIOD_A);
-	}
-
 	xc->arpeggio.count++;
 	xc->arpeggio.count %= xc->arpeggio.size;
 }
@@ -747,7 +746,7 @@ static void play_channel(struct context_data *ctx, int chn, int t)
 		return;
 
 #ifndef LIBXMP_CORE_PLAYER
-	play_extras(ctx, xc, chn, t == 0 && TEST(NEW_INS|NEW_NOTE));
+	play_extras(ctx, xc, chn, t);
 #endif
 
 	/* Do cut/retrig */
@@ -1159,6 +1158,7 @@ int xmp_play_buffer(xmp_context opaque, void *out_buffer, int size, int loop)
 	/* Reset internal state
 	 * Syncs buffer start with frame start */
 	if (out_buffer == NULL) {
+		p->loop_count = 0;
 		p->buffer_data.consumed = 0;
 		p->buffer_data.in_size = 0;
 		return 0;
