@@ -55,6 +55,8 @@ struct retrig_control {
 #define NOTE_FADEOUT	(1 << 0)
 #define NOTE_RELEASE	(1 << 1)
 #define NOTE_END	(1 << 2)
+#define NOTE_CUT	(1 << 3)
+#define NOTE_ENV_END	(1 << 4)
 
 #define IS_VALID_INSTRUMENT(x) ((uint32)(x) < mod->ins && mod->xxi[(x)].nsm > 0)
 #define IS_VALID_INSTRUMENT_OR_SFX(x) (((uint32)(x) < mod->ins && mod->xxi[(x)].nsm > 0) || (smix->ins > 0 && (uint32)(x) < mod->ins + smix->ins))
@@ -91,9 +93,22 @@ struct channel_data {
 	uint16 p_idx;		/* Pan envelope index */
 	uint16 f_idx;		/* Freq envelope index */
 
-	struct lfo vibrato;
-	struct lfo tremolo;
-	struct lfo panbrello;
+	struct {
+		struct lfo lfo;
+		int memory;
+	} vibrato;
+
+	struct {
+		struct lfo lfo;
+		int memory;
+	} tremolo;
+
+#ifndef LIBXMP_CORE_DISABLE_IT
+	struct {
+		struct lfo lfo;
+		int memory;
+	} panbrello;
+#endif
 
 	struct {
         	int8 val[16];	/* 16 for Smaksak MegaArps */
@@ -126,6 +141,11 @@ struct channel_data {
 		int memory;	/* Volume slide effect memory */
 	} vol;
 
+	struct {
+		int up_memory;	/* Fine volume slide up memory (XM) */
+		int down_memory;/* Fine volume slide up memory (XM) */
+	} fine_vol;
+
 	struct {
 		int slide;	/* Global volume slide value */
 		int fslide;	/* Fine global volume slide value */
@@ -151,6 +171,11 @@ struct channel_data {
 		int memory;	/* Tone portamento effect memory */
 	} porta;
 
+	struct {
+		int up_memory;	/* FT2 has separate memories for these */
+		int down_memory;/* cases (see Porta-LinkMem.xm) */
+	} fine_porta;
+
 	struct {
 		int val;	/* Current pan value */
 		int slide;	/* Pan slide value */
@@ -158,6 +183,24 @@ struct channel_data {
 		int memory;	/* Pan slide effect memory */
 	} pan;	
 
+	struct {
+		int speed;
+		int count;
+		int pos;
+	} invloop;
+
+#ifndef LIBXMP_CORE_DISABLE_IT
+	struct {
+		int slide;	/* IT tempo slide */
+	} tempo;
+
+	struct {
+		int cutoff;	/* IT filter cutoff frequency */
+		int resonance;	/* IT filter resonance */
+	} filter;
+
+#endif
+
 #ifndef LIBXMP_CORE_PLAYER
 	struct {
 		int slide;	/* PTM note slide amount */
@@ -165,24 +208,7 @@ struct channel_data {
 		int speed;	/* PTM note slide speed */
 		int count;	/* PTM note slide counter */
 	} noteslide;
-#endif
 
-	struct {
-		int slide;	/* IT tempo slide */
-	} tempo;
-
-	struct {
-		int speed;
-		int count;
-		int pos;
-	} invloop;
-
-	struct {
-		int cutoff;	/* IT filter cutoff frequency */
-		int resonance;	/* IT filter resonance */
-	} filter;
-
-#ifndef LIBXMP_CORE_PLAYER
 	void *extra;
 #endif
 
