@@ -59,60 +59,54 @@ static struct xmp_subinstrument *get_subinstrument(struct context_data *ctx,
 	return NULL;
 }
 
-static void reset_envelopes(struct context_data *ctx, struct channel_data *xc)
+static void reset_envelopes(struct context_data *ctx, struct channel_data *xc,
+				int force_cut)
 {
 	struct xmp_instrument *xxi;
 
 	xxi = get_instrument(ctx, xc->ins);
 
 	/* Reset envelope positions */
-	if (~xxi->aei.flg & XMP_ENVELOPE_CARRY) {
+	if (force_cut || (~xxi->aei.flg & XMP_ENVELOPE_CARRY)) {
 		xc->v_idx = 0;
 	}
-	if (~xxi->pei.flg & XMP_ENVELOPE_CARRY) {
+	if (force_cut || (~xxi->pei.flg & XMP_ENVELOPE_CARRY)) {
 		xc->p_idx = 0;
 	}
-	if (~xxi->fei.flg & XMP_ENVELOPE_CARRY) {
+	if (force_cut || (~xxi->fei.flg & XMP_ENVELOPE_CARRY)) {
 		xc->f_idx = 0;
 	}
+
+	RESET_NOTE(NOTE_ENV_END);
 }
 
 static void set_effect_defaults(struct context_data *ctx, int note,
 				struct xmp_subinstrument *sub,
 				struct channel_data *xc, int is_toneporta)
 {
-	struct module_data *m = &ctx->m;
-
 	if (sub != NULL && note >= 0) {
 		xc->finetune = sub->fin;
 		xc->gvl = sub->gvl;
 
 		if (sub->ifc & 0x80) {
 			xc->filter.cutoff = (sub->ifc - 0x80) * 2;
-		} else {
+		} /*else {
 			xc->filter.cutoff = 0xff;
-		}
+		} */
 
 		if (sub->ifr & 0x80) {
 			xc->filter.resonance = (sub->ifr - 0x80) * 2;
-		} else {
+		} /* else {
 			xc->filter.resonance = 0;
-		}
+		} */
 
 		set_lfo_depth(&xc->insvib.lfo, sub->vde);
 		set_lfo_rate(&xc->insvib.lfo, sub->vra >> 2);
 		set_lfo_waveform(&xc->insvib.lfo, sub->vwf);
 		xc->insvib.sweep = sub->vsw;
 
-		set_lfo_phase(&xc->vibrato, 0);
-		set_lfo_phase(&xc->tremolo, 0);
-
-		xc->porta.target = note_to_period(note,
-				xc->finetune, HAS_QUIRK(QUIRK_LINEAR),
-				xc->per_adj);
-		if (xc->period < 1 || !is_toneporta) {
-			xc->period = xc->porta.target;
-		}
+		set_lfo_phase(&xc->vibrato.lfo, 0);
+		set_lfo_phase(&xc->tremolo.lfo, 0);
 	}
 
 	xc->delay = 0;
@@ -124,6 +118,21 @@ static void set_effect_defaults(struct context_data *ctx, int note,
 	xc->arpeggio.size = 1;
 }
 
+static void set_period(struct context_data *ctx, int note,
+				struct xmp_subinstrument *sub,
+				struct channel_data *xc, int is_toneporta)
+{
+	struct module_data *m = &ctx->m;
+
+	if (sub != NULL && note >= 0) {
+		xc->porta.target = note_to_period(note, xc->finetune,
+				HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+		if (xc->period < 1 || !is_toneporta) {
+			xc->period = xc->porta.target;
+		}
+	}
+}
+
 
 #ifndef LIBXMP_CORE_PLAYER
 #define IS_TONEPORTA(x) ((x) == FX_TONEPORTA || (x) == FX_TONE_VSLIDE \
@@ -224,7 +233,7 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 
 	set_effect_defaults(ctx, note, sub, xc, is_toneporta);
 	if (e->ins && sub != NULL) {
-		reset_envelopes(ctx, xc);
+		reset_envelopes(ctx, xc, 0);
 	}
 
 	/* Process new volume */
@@ -236,6 +245,7 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 	/* Secondary effect handled first */
 	process_fx(ctx, xc, chn, e->note, e->f2t, e->f2p, 1);
 	process_fx(ctx, xc, chn, e->note, e->fxt, e->fxp, 0);
+	set_period(ctx, note, sub, xc, is_toneporta);
 
 	if (TEST(NEW_VOL))
 		use_ins_vol = 0;
@@ -417,18 +427,23 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	sub = get_subinstrument(ctx, xc->ins, xc->key);
 
 	set_effect_defaults(ctx, note, sub, xc, is_toneporta);
+
 	if (ins && sub != NULL) {
 		/* Reset envelopes on new instrument, see olympic.xm pos 10
 		 * But make sure we have an instrument set, see Letting go
 		 * pos 4 chn 20
 		 */
-		reset_envelopes(ctx, xc);
+		reset_envelopes(ctx, xc, 0);
 	}
 
 	/* Process new volume */
 	if (e->vol) {
 		xc->volume = e->vol - 1;
 		SET(NEW_VOL);
+		if (TEST_NOTE(NOTE_END)) {	/* m5v-nine.xm */
+			xc->fadeout = 0x10000;	/* OpenMPT NoteOff.xm */
+			RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
+		}
 	}
 
 	/* FT2: always reset sample offset */
@@ -437,6 +452,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	/* Secondary effect handled first */
 	process_fx(ctx, xc, chn, e->note, e->f2t, e->f2p, 1);
 	process_fx(ctx, xc, chn, e->note, e->fxt, e->fxp, 0);
+	set_period(ctx, note, sub, xc, is_toneporta);
 
 	if (TEST(NEW_VOL))
 		use_ins_vol = 0;
@@ -448,15 +464,22 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	if (note >= 0) {
 		xc->note = note;
 
-		/* (From Decibelter - Cosmic 'Wegian Mamas.xm p04 ch7)
-		 * We retrigger the sample only if we have a new note without
-		 * tone portamento, otherwise we won't play sweeps and loops
-		 * correctly.
+		/* From the OpenMPT test cases (3xx-no-old-samp.xm):
+		 * "An offset effect that points beyond the sample end should
+		 * stop playback on this channel."
 		 */
-		virt_voicepos(ctx, chn, xc->offset_val);
-		if (TEST(OFFSET))
-			xc->offset_val <<= 1;
-		RESET(OFFSET);
+
+		if (xc->offset_val >= mod->xxs[sub->sid].len) {
+			virt_resetchannel(ctx, chn);
+		} else {
+
+			/* (From Decibelter - Cosmic 'Wegian Mamas.xm p04 ch7)
+			 * We retrigger the sample only if we have a new note
+			 * without tone portamento, otherwise we won't play
+			 * sweeps and loops correctly.
+			 */
+			virt_voicepos(ctx, chn, xc->offset_val);
+		}
 	}
 
 	if (use_ins_vol) {
@@ -489,6 +512,10 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 		is_toneporta = 1;
 	}
 
+	if (virt_mapchannel(ctx, chn) < 0 && xc->ins != e->ins - 1) {
+		is_toneporta = 0;
+	}
+
 	/* Check instrument */
 
 	if (e->ins) {
@@ -571,7 +598,7 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 
 	set_effect_defaults(ctx, note, sub, xc, is_toneporta);
 	if (e->ins && sub != NULL) {
-		reset_envelopes(ctx, xc);
+		reset_envelopes(ctx, xc, 0);
 	}
 
 	/* Process new volume */
@@ -583,6 +610,7 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 	/* Secondary effect handled first */
 	process_fx(ctx, xc, chn, e->note, e->f2t, e->f2p, 1);
 	process_fx(ctx, xc, chn, e->note, e->fxt, e->fxp, 0);
+	set_period(ctx, note, sub, xc, is_toneporta);
 
 	if (TEST(NEW_VOL))
 		use_ins_vol = 0;
@@ -594,9 +622,6 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 	if (note >= 0) {
 		xc->note = note;
 		virt_voicepos(ctx, chn, xc->offset_val);
-		if (TEST(OFFSET))
-			xc->offset_val <<= 1;
-		RESET(OFFSET);
 	}
 
 	if (use_ins_vol) {
@@ -628,21 +653,21 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	int candidate_ins;
 	int reset_env;
 	int use_ins_vol;
-	unsigned char e_ins;
+	struct xmp_event ev;
 
-	e_ins = e->ins;
+	memcpy(&ev, e, sizeof (struct xmp_event));
 
 	/* Emulate Impulse Tracker "always read instrument" bug */
-	if (e_ins) {
+	if (ev.ins) {
 		xc->delayed_ins = 0;
-	} else if (e->note && xc->delayed_ins) {
-		e_ins = xc->delayed_ins;
+	} else if (ev.note && xc->delayed_ins) {
+		ev.ins = xc->delayed_ins;
 		xc->delayed_ins = 0;
 	}
 
 	xc->flags = 0;
 	note = -1;
-	key = e->note;
+	key = ev.note;
 	not_same_ins = 0;
 	new_invalid_ins = 0;
 	is_toneporta = 0;
@@ -651,7 +676,21 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	use_ins_vol = 0;
 	candidate_ins = xc->ins;
 
-	if (IS_TONEPORTA(e->fxt) || IS_TONEPORTA(e->f2t)) {
+	/* Notes with unmapped instruments are ignored */
+	if (ev.ins && ev.ins <= mod->ins && ev.note && ev.note <= XMP_MAX_KEYS) {
+		int ins = ev.ins - 1;
+		int key = ev.note - 1;
+
+		if (ins < mod->ins) {
+			int smp = mod->xxi[ins].map[key].ins;
+			if (smp == 0xff || smp >= mod->smp) {
+				candidate_ins = ins;
+				memset(&ev, 0, sizeof(struct xmp_event));
+			};
+		}
+	}
+
+	if (IS_TONEPORTA(ev.fxt) || IS_TONEPORTA(ev.f2t)) {
 		is_toneporta = 1;
 	}
 
@@ -659,16 +698,19 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		is_release = 1;
 	}
 
+	if (!HAS_QUIRK(QUIRK_VIRTUAL) && virt_mapchannel(ctx, chn) < 0) {
+		is_toneporta = 0;
+	}
+
 	/* Check instrument */
 
-	if (e_ins) {
-		int ins = e_ins - 1;
+	if (ev.ins) {
+		int ins = ev.ins - 1;
 
 		if (!is_release || (!is_toneporta || xc->ins != ins)) {
 			SET(NEW_INS);
 			use_ins_vol = 1;
 			reset_env = 1;
-			xc->fadeout = 0x10000;
 		}
 		xc->per_flags = 0;
 
@@ -684,7 +726,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 					key = xc->key + 1;
 				}
 			}
-			if (xc->ins != ins) {
+			if (xc->ins != ins && (!is_toneporta || !HAS_QUIRK(QUIRK_PRENV))) {
 				not_same_ins = 1;
 				candidate_ins = ins;
 				if (is_toneporta) {
@@ -714,7 +756,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			reset_env = 0;
 			use_ins_vol = 0;
 		} else if (key == XMP_KEY_CUT) {
-			SET_NOTE(NOTE_END);
+			SET_NOTE(NOTE_END | NOTE_CUT);
 			xc->period = 0;
 			virt_resetchannel(ctx, chn);
 		} else if (key == XMP_KEY_OFF) {
@@ -734,9 +776,6 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			} else {
 				key = 0;
 			}
-
-			if (HAS_QUIRK(QUIRK_PRENV) && e->ins)
-				reset_envelopes(ctx, xc);
 		}
 	}
 
@@ -748,7 +787,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 		if (sub != NULL) {
 			int transp = mod->xxi[candidate_ins].map[key].xpo;
-			int smp;
+			int smp, to;
 
 			note = key + sub->xpo + transp;
 			smp = sub->sid;
@@ -756,19 +795,18 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 				smp = -1;
 			}
 
-			if (smp >= 0 && smp < mod->smp) {
-				int to = virt_setpatch(ctx, chn, candidate_ins,
-					smp, note, sub->nna, sub->dct,
-					sub->dca);
+			to = virt_setpatch(ctx, chn, candidate_ins,
+				smp, note, sub->nna, sub->dct, sub->dca);
 
-				if (to < 0)
-					return -1;
+			if (to < 0)
+				return -1;
 
-				if (to != chn) {
-					copy_channel(p, to, chn);
-					p->xc_data[to].flags = 0;
-				}
+			if (to != chn) {
+				copy_channel(p, to, chn);
+				p->xc_data[to].flags = 0;
+			}
 
+			if (smp >= 0) {		/* Not sure if needed */
 				xc->smp = smp;
 			}
 		} else {
@@ -777,7 +815,26 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		}
 	}
 
+	/* Do after virtual channel copy */
+	if (is_toneporta) {
+		if (HAS_QUIRK(QUIRK_PRENV) && ev.ins) {
+			reset_envelopes(ctx, xc, 0);
+		}
+	}
+
+	/* Reset in case of new instrument and the previous envelope has
+	 * finished (OpenMPT test EnvReset.it). This must take place after
+	 * channel copies in case of NNA (see test/test.it)
+	 */
+	if (ev.ins && TEST_NOTE(NOTE_ENV_END)) {
+		reset_envelopes(ctx, xc, 1);
+	}
+
 	if (IS_VALID_INSTRUMENT(candidate_ins)) {
+		if (xc->ins != candidate_ins) {
+			/* Reset envelopes if instrument changes */
+			reset_envelopes(ctx, xc, 1);
+		}
 		xc->ins = candidate_ins;
 	}
 
@@ -788,16 +845,17 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		if (note >= 0) {
 			if (sub->pan >= 0)
 				xc->pan.val = sub->pan;
-			reset_envelopes(ctx, xc);
-		} else if (e_ins) {
+			reset_envelopes(ctx, xc, TEST_NOTE(NOTE_CUT));
+			RESET_NOTE(NOTE_CUT);
+		} else if (ev.ins) {
 			if (sub->pan >= 0)
 				xc->pan.val = sub->pan;
 		}
 	}
 	
 	/* Process new volume */
-	if (e->vol) {
-		xc->volume = e->vol - 1;
+	if (ev.vol) {
+		xc->volume = ev.vol - 1;
 		SET(NEW_VOL);
 	}
 
@@ -807,8 +865,9 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	/* According to Storlek test 25, Impulse Tracker handles the volume
 	 * column effects last.
 	 */
-	process_fx(ctx, xc, chn, e->note, e->fxt, e->fxp, 0);
-	process_fx(ctx, xc, chn, e->note, e->f2t, e->f2p, 1);
+	process_fx(ctx, xc, chn, ev.note, ev.fxt, ev.fxp, 0);
+	process_fx(ctx, xc, chn, ev.note, ev.f2t, ev.f2p, 1);
+	set_period(ctx, note, sub, xc, is_toneporta);
 
 	if (TEST(NEW_VOL)) {
 		use_ins_vol = 0;
@@ -821,13 +880,12 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	if (note >= 0) {
 		xc->note = note;
 		virt_voicepos(ctx, chn, xc->offset_val);
-		if (TEST(OFFSET))
-			xc->offset_val <<= 1;
-		RESET(OFFSET);
 	}
 
 	if (reset_env) {
 		RESET_NOTE(NOTE_RELEASE | NOTE_FADEOUT);
+		/* Set after copying to new virtual channel (see ambio.it) */
+		xc->fadeout = 0x10000;
 	}
 
 	if (use_ins_vol) {
@@ -853,6 +911,7 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 	int new_invalid_ins = 0;
 	int is_toneporta;
 	int use_ins_vol;
+	int finetune;
 
 	xc->flags = 0;
 	note = -1;
@@ -906,6 +965,10 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 		if (e->note == XMP_KEY_OFF) {
 			SET_NOTE(NOTE_RELEASE);
 			use_ins_vol = 0;
+		} else if (e->note == XMP_KEY_CUT) {
+			SET_NOTE(NOTE_END);
+			xc->period = 0;
+			virt_resetchannel(ctx, chn);
 		} else if (!is_toneporta && IS_VALID_INSTRUMENT(xc->ins)) {
 			struct xmp_instrument *xxi = &mod->xxi[xc->ins];
 
@@ -948,9 +1011,15 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 
 	sub = get_subinstrument(ctx, xc->ins, xc->key);
 
+	/* Keep effect-set finetune if no instrument set */
+	finetune = xc->finetune;
 	set_effect_defaults(ctx, note, sub, xc, is_toneporta);
+	if (!e->ins) {
+		xc->finetune = finetune;
+	}
+
 	if (e->ins && sub != NULL) {
-		reset_envelopes(ctx, xc);
+		reset_envelopes(ctx, xc, 0);
 	}
 
 	/* Process new volume */
@@ -962,6 +1031,7 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 	/* Secondary effect handled first */
 	process_fx(ctx, xc, chn, e->note, e->f2t, e->f2p, 1);
 	process_fx(ctx, xc, chn, e->note, e->fxt, e->fxp, 0);
+	set_period(ctx, note, sub, xc, is_toneporta);
 
 	if (TEST(NEW_VOL)) {
 		use_ins_vol = 0;
@@ -973,11 +1043,7 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 
 	if (note >= 0) {
 		xc->note = note;
-
 		virt_voicepos(ctx, chn, xc->offset_val);
-		if (TEST(OFFSET) && p->flags & XMP_FLAGS_FX9BUG)
-			xc->offset_val <<= 1;
-		RESET(OFFSET);
 	}
 
 	if (use_ins_vol) {
@@ -1057,8 +1123,10 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 	}
 
 	set_effect_defaults(ctx, note, sub, xc, 0);
+	set_period(ctx, note, sub, xc, 0);
+
 	if (e->ins && sub != NULL) {
-		reset_envelopes(ctx, xc);
+		reset_envelopes(ctx, xc, 0);
 	}
 
 	xc->volume = e->vol - 1;
